#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

#include "mathfunc.h"
#include "brkpnts.h"

/* ----------------------------------------------------------------------------
 * NOTE:  the range limits for the Bernoulli and Aux functions were generated
 *	  by the "brkpnts" program and stored in brkpnts.h.
 * --------------------------------------------------------------------------*/

/* ----------------------------------------------------------------------------
 * bern:  This function returns the Bernoulli function of the argument.	 To
 * avoid under and overflows this function is defined by equivalent or approx-
 * imate functions depending upon the value of the argument.
 *
 *					  x
 *				B(x) = -------
 *				       e^x - 1
 *
 */
REAL bern ( REAL x )
{
  REAL y;

  if (x <= BP0_BERN)
  { return(-x); }
  else if (x <	BP1_BERN)
  { return(x / (exp(x) - 1.0)); }
  else if (x <= BP2_BERN)
  { return(1.0 - x/2.0 * (1.0 - x/6.0 * (1.0 - x*x/60.0))); }
  else if (x <	BP3_BERN)
  { y = exp(-x);   return((x * y) / (1.0 - y)); }
  else if (x <	BP4_BERN)
  { return(x * exp(-x)); }
  else { return(0.0); }

} /* bern */


/* ----------------------------------------------------------------------------
 * pd1bern:  This function returns the total derivative of the Bernoulli
 * function with respect to the argument.  To avoid under and overflows this
 * function is defined by equivalent or approximate functions depending upon
 * the value of the argument.
 *
 *			   d	    (1-x)*e^x - 1
 *			   --B(x) = -------------
 *			   dx	     (e^x - 1)^2
 */
REAL pd1bern ( REAL x )
{
  REAL y, z;

  if (x <= BP0_DBERN)
  { return(-1.0); }
  else if (x <= BP1_DBERN)
  { return((1.0 - x)*exp(x) - 1.0); }
  else if (x <	BP2_DBERN)
  { y = exp(x); z = y - 1.0; return(((1.0 - x)*y - 1.0)/(z*z)); }
  else if (x <= BP3_DBERN)
  { return(-0.5 + x/6.0 * (1.0 - x*x/30.0)); }
  else if (x <	BP4_DBERN)
  { y = exp(-x); z = 1 - y;  return(((1.0 - x)*y - y*y )/(z*z)); }
  else if (x <	BP5_DBERN)
  { y = exp(-x); return((1.0 - x)*y - y*y); }
  else { return(0.0); }

} /* pd1bern */


/* ----------------------------------------------------------------------------
 * aux1:  This function returns the aux1 function.  To avoid under and over-
 * flows this function is defined by equivalent or approximate functions
 * depending upon the value of the argument.
 *
 *					 x
 *			   Aux1(x) =  -------
 *				      sinh(x)
 */
REAL aux1 ( REAL x )
{
  if	  (x < -BP0_MISC) x = -BP0_MISC;
  else if (x >	BP0_MISC) x =  BP0_MISC;

  if (x <= BP0_AUX1)
  { return(x / sinh(x)); }
  else if (x <= BP1_AUX1)
  { return(1 - x*x/6.0*(1.0 - 7.0*x*x/60.0)); }
  else { return(x / sinh(x)); }

} /* aux1 */


/* ----------------------------------------------------------------------------
 * pd1aux1:  This function returns the total derivative of the aux1 function.
 * To avoid under and overflows this function is defined by equivalent or
 * approximate functions depending upon the value of the argument.
 *
 *		      d		  sinh(x) - x*cosh(x)
 *		      --Aux1(x) = -------------------
 *		      dx	      (sinh(x))^2
 */
REAL pd1aux1 ( REAL x )
{
  REAL y;

  if	  (x < -BP0_MISC) x = -BP0_MISC;
  else if (x >	BP0_MISC) x =  BP0_MISC;

  if (x <= BP0_DAUX1)
  { y = sinh(x); return((y - x*cosh(x))/(y*y)); }
  else if (x <= BP1_DAUX1)
  { return(-x/3.0*(1.0 - 7.0*x*x/30.0)); }
  else { y = sinh(x); return((y - x*cosh(x))/(y*y)); }

} /* pd1aux1 */


/* ----------------------------------------------------------------------------
 * aux2:  This function returns the aux2 function.  To avoid under and over-
 * flows this function is defined by equivalent or approximate functions
 * depending upon the value of the argument.
 *
 *					     1
 *				Aux2(x) = -------
 *					  1 + e^x
 */
REAL aux2 ( REAL x )
{
  if (x <= BP0_AUX2)
  { return(1.0); }
  else if (x <= BP1_AUX2)
  { return(1.0 / (1.0 + exp(x))); }
  else if (x <= BP2_AUX2)
  { return(exp(-x)); }
  else { return(0.0); }

} /* aux2 */


/* ----------------------------------------------------------------------------
 * pd1aux2:  This function returns the total derivative of the aux2 function.
 * To avoid under and overflows this function is defined by equivalent or
 * approximate functions depending upon the value of the argument.
 *
 *			   d		 - e^x
 *			   --Aux2(x) = -----------
 *			   dx	       (1 + e^x)^2
 */
REAL pd1aux2 ( REAL x )
{
  REAL y,z;

  if (x <= BP0_DAUX2)
  { return(0.0); }
  else if (x <= BP1_DAUX2)
  { return(-exp(x)); }
  else if (x <= BP2_DAUX2)
  { y = exp(x); z = y + 1.0; return(-y/(z*z)); }
  else if (x <= BP3_DAUX2)
  { return(-exp(-x)); }
  else { return(0.0); }

} /* pd1aux2 */



