#include "solve.h"
#include "physical.h"
#include "mathfunc.h"
#include <math.h>
//------------------------------------------------------------------------------------
double Solver::VoltageWave()
{
 //if(t<2e5) return Vapp*t/2e5;
 //else return Vapp;
 
 return Vapp;
 //return 1+0.05*sin(3.1415926*t);
 
}


//--------------------------------------------------------
void Solver::init_diode()
{
	int i;
	double N;
	double Na  = -1.00e+18/1e15;
        double Nd  = 1.00e+14/1e15;
	for(i=0;i<nodes;i++)
	{
		double x=i*h;
	 	N= Na-(Na-Nd)*ngdep(x,2*100*h,h);
		//N=1.0 * (i>nodes/2)?-1:1;
	 	if(N>0)
	 	{
	  		Q[i].Nd=N;	
	  		Q[i].Na=0;	
	  		Q[i].n=(fabs(N)+sqrt(N*N+4*nie))/2.0;
	  		Q[i].p=nie*nie/Q[i].n;
	 	} 
	 	else
	 	 {
	  		Q[i].Na=fabs(N);	
	  		Q[i].Nd=0;		
	  		Q[i].p=(fabs(N)+sqrt(N*N+4*nie))/2.0;
	  		Q[i].n=nie*nie/Q[i].p;
	 	} 
	 
	 	Q[i].P = Vbi(Q[i].Na,Q[i].Nd);
	 	Q[i].E_field = 0;
	}
} 



//----------------------------------------------------------
PetscErrorCode FormFunction(SNES snes,Vec x,Vec f,void *dummy)
{
   Solver *psolver = (Solver*)dummy;
   PetscScalar    *xx,*ff;
   double Vt = k*Tl/e;
   /*
      Get pointers to vector data.
        - For default PETSc vectors, VecGetArray() returns a pointer to
          the data array.  Otherwise, the routine is implementation dependent.
        - You MUST call VecRestoreArray() when you no longer need access to
          the array.
   */
   VecGetArray(x,&xx);
   VecGetArray(f,&ff);
   
   /*
      Compute function
   */
   for(int i=0;i<nodes; i++)
   {
   	if(i == 0) // cathode  
   	{	ff[0] = xx[0] - Vbi(psolver->Q[0].Na,psolver->Q[0].Nd);  //potential equal to 0
   		double electron_density = (fabs(psolver->Q[0].Nd)+sqrt(psolver->Q[0].Nd*psolver->Q[0].Nd+4*nie))/2.0;
   		double hole_density = nie*nie/electron_density;
   	        ff[1] = xx[1] - electron_density;  //electron density
   	        ff[2] = xx[2] - hole_density; //hole density
   	}
   	else if(i == nodes-1) //anode 
   	{
   		ff[3*i]   = xx[3*i] - psolver->VoltageWave() - Vbi(psolver->Q[nodes-1].Na,psolver->Q[nodes-1].Nd); //potential equal to voltage source
   		double hole_density = (fabs(psolver->Q[i].Na)+sqrt(psolver->Q[i].Na*psolver->Q[i].Na+4*nie))/2.0;
   		double electron_density = nie*nie/hole_density;
   		ff[3*i+2] = xx[3*i+2] - hole_density; //hole density  
   		ff[3*i+1] = xx[3*i+1] - electron_density; //electron density
   	}
   	else
   	{
   		double Vi = xx[3*i+0];     //potential of node i
   		double Vr = xx[3*(i+1)+0]; //potential of node i+1
   		double Vl = xx[3*(i-1)+0]; //potential of node i-1
   		
   		double ni = xx[3*i+1];	       //electron density of node i
   		double nr = xx[3*(i+1)+1];     //electron density of node i+1
   		double nl = xx[3*(i-1)+1];     //electron density of node i-1
   		
   		double pi = xx[3*i+2];	       //hole density of node i
   		double pr = xx[3*(i+1)+2];     //hole density of node i+1
   		double pl = xx[3*(i-1)+2];     //hole density of node i-1
   		
   		double hEl = -(Vi-Vl)/h;	//electrical field value in half point i-1/2
   		double hEr = -(Vr-Vi)/h;	//electrical field value in half point i+1/2
   		
   		double hnl = nl*aux2((Vl-Vi)/(2*Vt)) + ni*aux2((Vi-Vl)/(2*Vt)); //electron density of half point i-1/2
   		double hnr = ni*aux2((Vi-Vr)/(2*Vt)) + nr*aux2((Vr-Vi)/(2*Vt)); //electron density of half point i+1/2
   		double hdndxl = aux1((Vl-Vi)/(2*Vt))*(ni-nl)/h; // electron density gradient of half point i-1/2
   		double hdndxr = aux1((Vi-Vr)/(2*Vt))*(nr-ni)/h; // electron density gradient of half point i+1/2
   		
   		double hpl = pl*aux2((Vi-Vl)/(2*Vt)) + pi*aux2((Vl-Vi)/(2*Vt)); //hole density of half point i-1/2
   		double hpr = pi*aux2((Vr-Vi)/(2*Vt)) + pr*aux2((Vi-Vr)/(2*Vt)); //hole density of half point i+1/2
   		double hdpdxl = aux1((Vl-Vi)/(2*Vt))*(pi-pl)/h; // hole density gradient of half point i-1/2
   		double hdpdxr = aux1((Vi-Vr)/(2*Vt))*(pr-pi)/h; // hole density gradient of half point i+1/2
   		
   		double hmunr =  (mu0n(psolver->Q[i].Na,psolver->Q[i].Nd)+mu0n(psolver->Q[i+1].Na,psolver->Q[i+1].Nd))/2;
   		double hmunl =  (mu0n(psolver->Q[i].Na,psolver->Q[i].Nd)+mu0n(psolver->Q[i-1].Na,psolver->Q[i-1].Nd))/2;
   		double hmupr =  (mu0p(psolver->Q[i].Na,psolver->Q[i].Nd)+mu0p(psolver->Q[i+1].Na,psolver->Q[i+1].Nd))/2;
   		double hmupl =  (mu0p(psolver->Q[i].Na,psolver->Q[i].Nd)+mu0p(psolver->Q[i-1].Na,psolver->Q[i-1].Nd))/2;
   		   		
   		double U = R(psolver->Q[i].Na,psolver->Q[i].Nd,pi,ni);
   		
   		ff[3*i]   =  (Vl - 2*Vi + Vr)/(h*h) + e/eps*(pi-ni+psolver->Q[i].Nd-psolver->Q[i].Na);
   		
   		ff[3*i+1] = -(ni-psolver->Q[i].n)/psolver->dt + ( hmunr*(hEr*hnr + Vt*hdndxr)-hmunl*(hEl*hnl + Vt*hdndxl) )/h - U;
   		
   		ff[3*i+2] = -(pi-psolver->Q[i].p)/psolver->dt - ( hmupr*(hEr*hpr - Vt*hdpdxr)-hmupl*(hEl*hpl - Vt*hdpdxl) )/h - U;
   		
   		
   	}
   }			  
  	
   /*
      Restore vectors
   */
   VecRestoreArray(x,&xx);
   VecRestoreArray(f,&ff);
  
   return 0;
}

PetscErrorCode FormJacobian(SNES snes,Vec x,Mat *jac,Mat *B,MatStructure *flag,void *dummy)
{
   PetscScalar    *xx,A[9];
   PetscInt       index[3],col[3];
   Solver *psolver = (Solver*)dummy;
   double Vt = k*Tl/e;
   /*
      Get pointer to vector data
   */
   VecGetArray(x,&xx);

   /*
      Compute Jacobian entries and insert into matrix.
   */
   for(int i=0;i<nodes; i++)
   {
   	if(i == 0 ||i == nodes-1) // cathode or anode
   	{
   		A[0] = 1;  A[1] = 0;  A[2] = 0;
   		A[3] = 0;  A[4] = 1;  A[5] = 0;
   	        A[6] = 0;  A[7] = 0;  A[8] = 1;
   	        index[0] = 3*i;
   		index[1] = 3*i+1;
   		index[2] = 3*i+2;
   	
   		MatSetValues(*jac,3,index,3,index,A,INSERT_VALUES);
   	}
   	else         //inner point 
   	{
   		double Vi = xx[3*i+0];     //potential of node i
   		double Vr = xx[3*(i+1)+0]; //potential of node i+1
   		double Vl = xx[3*(i-1)+0]; //potential of node i-1
   		
   		double ni = xx[3*i+1];	       //electron density of node i
   		double nr = xx[3*(i+1)+1];     //electron density of node i+1
   		double nl = xx[3*(i-1)+1];     //electron density of node i-1
   		
   		double pi = xx[3*i+2];	       //hole density of node i
   		double pr = xx[3*(i+1)+2];     //hole density of node i+1
   		double pl = xx[3*(i-1)+2];     //hole density of node i-1
   		//-----------------------------------------------------------------------------------------------
   		double hEl = -(Vi-Vl)/h;	//electrical field value in half point i-1/2
   		double hEr = -(Vr-Vi)/h;	//electrical field value in half point i+1/2
   		
   		double hnl = nl*aux2((Vl-Vi)/(2*Vt)) + ni*aux2((Vi-Vl)/(2*Vt)); //electron density of half point i-1/2
   		double hnr = ni*aux2((Vi-Vr)/(2*Vt)) + nr*aux2((Vr-Vi)/(2*Vt)); //electron density of half point i+1/2
   		double hdndxl = aux1((Vl-Vi)/(2*Vt))*(ni-nl)/h; // electron density gradient of half point i-1/2
   		double hdndxr = aux1((Vi-Vr)/(2*Vt))*(nr-ni)/h; // electron density gradient of half point i+1/2
   		
   		double hpl = pl*aux2((Vi-Vl)/(2*Vt)) + pi*aux2((Vl-Vi)/(2*Vt)); //hole density of half point i-1/2
   		double hpr = pi*aux2((Vr-Vi)/(2*Vt)) + pr*aux2((Vi-Vr)/(2*Vt)); //hole density of half point i+1/2
   		double hdpdxl = aux1((Vl-Vi)/(2*Vt))*(pi-pl)/h; // hole density gradient of half point i-1/2
   		double hdpdxr = aux1((Vi-Vr)/(2*Vt))*(pr-pi)/h; // hole density gradient of half point i+1/2
   		
   		double hmunr =  (mu0n(psolver->Q[i].Na,psolver->Q[i].Nd)+mu0n(psolver->Q[i+1].Na,psolver->Q[i+1].Nd))/2;
   		double hmunl =  (mu0n(psolver->Q[i].Na,psolver->Q[i].Nd)+mu0n(psolver->Q[i-1].Na,psolver->Q[i-1].Nd))/2;
   		double hmupr =  (mu0p(psolver->Q[i].Na,psolver->Q[i].Nd)+mu0p(psolver->Q[i+1].Na,psolver->Q[i+1].Nd))/2;
   		double hmupl =  (mu0p(psolver->Q[i].Na,psolver->Q[i].Nd)+mu0p(psolver->Q[i-1].Na,psolver->Q[i-1].Nd))/2;
   		//-----------------------------------------------------------------------------------------------
   		double d_hEl_dVi = -1/h;
   		double d_hEl_dVl =  1/h;
   		double d_hEr_dVi =  1/h;
   		double d_hEr_dVr = -1/h;
   		
   		double d_hnr_dVi =   ni*pd1aux2((Vi-Vr)/(2*Vt))/(2*Vt) - nr*pd1aux2((Vr-Vi)/(2*Vt))/(2*Vt);
   		double d_hnr_dVr = - ni*pd1aux2((Vi-Vr)/(2*Vt))/(2*Vt) + nr*pd1aux2((Vr-Vi)/(2*Vt))/(2*Vt);
   		double d_hnl_dVi = - nl*pd1aux2((Vl-Vi)/(2*Vt))/(2*Vt) + ni*pd1aux2((Vi-Vl)/(2*Vt))/(2*Vt);
   		double d_hnl_dVl =   nl*pd1aux2((Vl-Vi)/(2*Vt))/(2*Vt) - ni*pd1aux2((Vi-Vl)/(2*Vt))/(2*Vt);
   		
   		double d_hpr_dVi = -pi*pd1aux2((Vr-Vi)/(2*Vt))/(2*Vt) + pr*pd1aux2((Vi-Vr)/(2*Vt))/(2*Vt);
   		double d_hpr_dVr =  pi*pd1aux2((Vr-Vi)/(2*Vt))/(2*Vt) - pr*pd1aux2((Vi-Vr)/(2*Vt))/(2*Vt);
   		double d_hpl_dVi =  pl*pd1aux2((Vi-Vl)/(2*Vt))/(2*Vt) - pi*pd1aux2((Vl-Vi)/(2*Vt))/(2*Vt);
   		double d_hpl_dVl = -pl*pd1aux2((Vi-Vl)/(2*Vt))/(2*Vt) + pi*pd1aux2((Vl-Vi)/(2*Vt))/(2*Vt);
   		//-----------------------------------------------------------------------------------------------
   		double d_hEnr_dVi = d_hEr_dVi*hnr+hEr*d_hnr_dVi;
   		double d_hEnr_dVr = d_hEr_dVr*hnr+hEr*d_hnr_dVr;
   		double d_hEnr_dni = hEr*aux2((Vi-Vr)/(2*Vt));
   		double d_hEnr_dnr = hEr*aux2((Vr-Vi)/(2*Vt));
   		
   		double d_hEnl_dVi = d_hEl_dVi*hnl + hEl*d_hnl_dVi;
   		double d_hEnl_dVl = d_hEl_dVl*hnl + hEl*d_hnl_dVl;
   		double d_hEnl_dni = hEl*aux2((Vi-Vl)/(2*Vt));
   		double d_hEnl_dnl = hEl*aux2((Vl-Vi)/(2*Vt));
   		
   		double d_hdndxr_dVi =  pd1aux1((Vi-Vr)/(2*Vt))/(2*Vt)*(nr-ni)/h;
   		double d_hdndxr_dVr = -pd1aux1((Vi-Vr)/(2*Vt))/(2*Vt)*(nr-ni)/h;
   		double d_hdndxr_dni = -aux1((Vi-Vr)/(2*Vt))/h;
   		double d_hdndxr_dnr =  aux1((Vi-Vr)/(2*Vt))/h;
   		
   		double d_hdndxl_dVi = -pd1aux1((Vl-Vi)/(2*Vt))/(2*Vt)*(ni-nl)/h;
   		double d_hdndxl_dVl =  pd1aux1((Vl-Vi)/(2*Vt))/(2*Vt)*(ni-nl)/h;
   		double d_hdndxl_dni =  aux1((Vl-Vi)/(2*Vt))/h;
   		double d_hdndxl_dnl = -aux1((Vl-Vi)/(2*Vt))/h;
   		//------------------------------------------------------------------------------------------------
   		
   		double d_hEpr_dVi =  d_hEr_dVi*hpr + hEr*d_hpr_dVi;
   		double d_hEpr_dVr =  d_hEr_dVr*hpr + hEr*d_hpr_dVr;		     
   		double d_hEpr_dpi =  hEr*aux2((Vr-Vi)/(2*Vt));
   		double d_hEpr_dpr =  hEr*aux2((Vi-Vr)/(2*Vt));
   		   		
   		double d_hEpl_dVi =  d_hEl_dVi*hpl + hEl*d_hpl_dVi;
   		double d_hEpl_dVl =  d_hEl_dVl*hpl + hEl*d_hpl_dVl;		     
   		double d_hEpl_dpi =  hEl*aux2((Vl-Vi)/(2*Vt));
   		double d_hEpl_dpl =  hEl*aux2((Vi-Vl)/(2*Vt));
   		
   		double d_hdpdxr_dVi =  pd1aux1((Vi-Vr)/(2*Vt))/(2*Vt)*(pr-pi)/h;
   		double d_hdpdxr_dVr = -pd1aux1((Vi-Vr)/(2*Vt))/(2*Vt)*(pr-pi)/h;
   		double d_hdpdxr_dpi = -aux1((Vi-Vr)/(2*Vt))/h;
   		double d_hdpdxr_dpr =  aux1((Vi-Vr)/(2*Vt))/h;
   		
   		double d_hdpdxl_dVi = -pd1aux1((Vl-Vi)/(2*Vt))/(2*Vt)*(pi-pl)/h;
   		double d_hdpdxl_dVl =  pd1aux1((Vl-Vi)/(2*Vt))/(2*Vt)*(pi-pl)/h;
   		double d_hdpdxl_dpi =  aux1((Vl-Vi)/(2*Vt))/h;
   		double d_hdpdxl_dpl = -aux1((Vl-Vi)/(2*Vt))/h;
   		//------------------------------------------------------------------------------------------------
    		
   		
   		//fun(0) is the poisson's equation
   		//fun(1) is the continuous equation of electron
   		//fun(2) is the continuous equation of hole
   		
   		//df(x)i/dx(i)     
   		A[0] =  -2.0/(h*h);    //dfun(0)/dP(i) 
   		A[1] =  -e/eps;              //dfun(0)/dn(i) 
   		A[2] =   e/eps;              //dfun(0)/dp(i)
   		
   		A[3] =  (hmunr*(d_hEnr_dVi+Vt*d_hdndxr_dVi)-hmunl*(d_hEnl_dVi+Vt*d_hdndxl_dVi))/h;           //dfun(1)/dP(i) 
   		A[4] =  -1/psolver->dt + (hmunr*(d_hEnr_dni+Vt*d_hdndxr_dni)-hmunl*(d_hEnl_dni+Vt*d_hdndxl_dni))/h;    //dfun(1)/dn(i)          
   		A[5] =  0;              				       				     //dfun(1)/dp(i)                  
   		
   		A[6] =  -(hmupr*(d_hEpr_dVi-Vt*d_hdpdxr_dVi)-hmupl*(d_hEpl_dVi-Vt*d_hdpdxl_dVi))/h;          //dfun(2)/dP(i)      
   		A[7] =  0;              				          			     //dfun(2)/dn(i) 
   		A[8] =  -1/psolver->dt - (hmupr*(d_hEpr_dpi-Vt*d_hdpdxr_dpi)-hmupl*(d_hEpl_dpi-Vt*d_hdpdxl_dpi))/h;    //dfun(2)/dp(i) 
   		
   		
   		index[0] = 3*i;
   		index[1] = 3*i+1;
   		index[2] = 3*i+2;
   	
   		MatSetValues(*jac,3,index,3,index,A,INSERT_VALUES);
   		
   		//df(x)i/dx(i+1)
   		A[0] =  1.0/(h*h);      //dfun(0)/dP(i+1) 
   		A[1] =  0;              //dfun(0)/dn(i+1) 
   		A[2] =  0;              //dfun(0)/dp(i+1)
   		
   		A[3] =  hmunr*(d_hEnr_dVr+Vt*d_hdndxr_dVr)/h;    //dfun(1)/dP(i+1)
   		A[4] =  hmunr*(d_hEnr_dnr+Vt*d_hdndxr_dnr)/h;    //dfun(1)/dn(i+1)
   		A[5] =   0;              		         //dfun(1)/dp(i+1)                  
   		
   		A[6] =  -hmupr*(d_hEpr_dVr-Vt*d_hdpdxr_dVr)/h;  //dfun(2)/dP(i+1)
   		A[7] =   0;              		        //dfun(2)/dn(i+1) 
   		A[8] =  -hmupr*(d_hEpr_dpr-Vt*d_hdpdxr_dpr)/h;  //dfun(2)/dp(i+1) 

   		
   		col[0] = index[0]+3;
   		col[1] = index[1]+3;
   		col[2] = index[2]+3;
   		
   	
   		MatSetValues(*jac,3,index,3,col,A,INSERT_VALUES);
   		
   		//df(x)i/dx(i-1)
   		A[0] =  1.0/(h*h);    //dfun(0)/dP(i-1) 
   		A[1] =  0;              //dfun(0)/dn(i-1) 
   		A[2] =  0;              //dfun(0)/dp(i-1)
   		A[3] =  -hmunl*(d_hEnl_dVl+Vt*d_hdndxl_dVl)/h;    //dfun(1)/dP(i-1) 
   		A[4] =  -hmunl*(d_hEnl_dnl+Vt*d_hdndxl_dnl)/h;    //dfun(1)/dn(i-1)          
   		A[5] =   0;              		          //dfun(1)/dp(i-1)                  
   		
   		A[6] =   hmupl*(d_hEpl_dVl-Vt*d_hdpdxl_dVl)/h;   //dfun(2)/dP(i-1)      
   		A[7] =   0;              		         //dfun(2)/dn(i-1) 
   		A[8] =   hmupl*(d_hEpl_dpl-Vt*d_hdpdxl_dpl)/h;   //dfun(2)/dp(i-1) 
   		
   		
   		col[0] = index[0]-3;
   		col[1] = index[1]-3;
   		col[2] = index[2]-3;
   	
   		MatSetValues(*jac,3,index,3,col,A,INSERT_VALUES);
 
   	}
   	
   
   }	
  
   
   *flag = SAME_NONZERO_PATTERN;

   /*
      Restore vector
   */
   VecRestoreArray(x,&xx);

   /*
      Assemble matrix
   */
   MatAssemblyBegin(*jac,MAT_FINAL_ASSEMBLY);
   MatAssemblyEnd(*jac,MAT_FINAL_ASSEMBLY);

   return 0;
}


void Solver::init_solver()
{
   
   /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      Create nonlinear solver context
      - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */

   SNESCreate(PETSC_COMM_WORLD,&snes);

   /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      Create matrix and vector data structures; set corresponding routines
      - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */

   /*
      Create vectors for solution and nonlinear function
   */
   VecCreateSeq(PETSC_COMM_SELF,3*nodes,&x);
   VecDuplicate(x,&r);

   /*
      Create Jacobian matrix data structure
   */
   //MatCreate(PETSC_COMM_SELF,PETSC_DECIDE,PETSC_DECIDE,3*nodes,3*nodes,&J);
   MatCreateSeqAIJ(PETSC_COMM_SELF,3*nodes,3*nodes,9,PETSC_NULL,&J);
   MatSetFromOptions(J);

   /* 
     Set function evaluation routine and vector.
   */
   SNESSetFunction(snes,r,FormFunction,this);

   /* 
    Set Jacobian matrix data structure and Jacobian evaluation routine
   */
   SNESSetJacobian(snes,J,J,FormJacobian,this);
 

   /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      Customize nonlinear solver; set runtime options
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */

   /* 
      Set linear solver defaults for this problem. By extracting the
      KSP, KSP, and PC contexts from the SNES context, we can then
      directly call any KSP, KSP, and PC routines to set various options.
    */
   SNESGetKSP(snes,&ksp);
   
   KSPGetPC(ksp,&pc);
   KSPSetType(ksp,KSPGMRES);
   KSPGMRESSetRestart(ksp,150);
   PCSetType(pc,PCILU);
   PCFactorReorderForNonzeroDiagonal(pc,1e-14);
   PCFactorSetShiftNonzero(pc,1e-12);
   //KSPSetTolerances(ksp,1e-8,PETSC_DEFAULT,PETSC_DEFAULT,200);

   /* 
      Set SNES/KSP/KSP/PC runtime options, e.g.,
          -snes_view -snes_monitor -ksp_type <ksp> -pc_type <pc>
      These options will override those specified above as long as
      SNESSetFromOptions() is called _after_ any other customization
      routines.
   */
   SNESSetFromOptions(snes);
   double Vt = k*Tl/e; 
   SNESSetTolerances(snes,1e-6*Vt,PETSC_DEFAULT,PETSC_DEFAULT,250,10000);

   /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      Evaluate initial guess; then solve nonlinear system
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */

   /*
      Note: The user should initialize the vector, x, with the initial guess
      for the nonlinear solver prior to calling SNESSolve().  In particular,
      to employ an initial guess of zero, the user should explicitly set
      this vector to zero by calling VecSet().
   */
   double init_value[3];
   int    index[3];
   for(int i=0;i<nodes; i++)
   {
   	index[0]=3*i;
   	index[1]=3*i+1;
   	index[2]=3*i+2;
   	init_value[0] = Q[i].P;
   	init_value[1] = Q[i].n;
   	init_value[2] = Q[i].p;	
   	VecSetValues(x,3,index,init_value,INSERT_VALUES);
   }	
   VecAssemblyBegin(x);
   VecAssemblyEnd(x);

}

void Solver::do_solver()
{
   PetscInt       its;
   PetscReal      norm;
   SNESConvergedReason reason;
   SNESSolve(snes,PETSC_NULL,x);
   SNESGetConvergedReason(snes,&reason);
   SNESGetIterationNumber(snes,&its);
 
   //VecView(r,PETSC_VIEWER_STDOUT_WORLD);
   VecNorm(r,NORM_1,&norm); 
   printf("\tnumber of iterations=%D norm=%e  ConvergedReason %d\n",its,norm,reason);
   PetscScalar    *xx;
   VecGetArray(x,&xx);
   for(int i=0;i<nodes; i++)
   {
   	
   	Q[i].P     = xx[3*i];
   	Q[i].n     = xx[3*i+1];
   	Q[i].p     = xx[3*i+2];
   
   }	
   VecRestoreArray(x,&xx);
   /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      Free work space.  All PETSc objects should be destroyed when they
      are no longer needed.
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
}



void Solver::destroy_solver()
{
   VecDestroy(x); VecDestroy(r);
   MatDestroy(J); SNESDestroy(snes);
 
}


double Solver::current_cathode()
{
	double Vt = k*Tl/e;
	
	double Vi = Q[0].P;     //potential of node 0
   	double Vr = Q[1].P; 	//potential of node 1
	
	double ni = Q[0].n;	//electron density of node 0
   	double nr = Q[1].n;     //electron density of node 1
   
   	double pi = Q[0].p;	//hole density of node 0
   	double pr = Q[1].p;     //hole density of node 1
   	
   	double hEr = -(Vr-Vi)/h;	//electrical field value in half point 1/2
   	
   	double hnr = ni*aux2((Vi-Vr)/(2*Vt)) + nr*aux2((Vr-Vi)/(2*Vt)); 
   	double hdndxr = aux1((Vi-Vr)/(2*Vt))*(nr-ni)/h;
   	
   	double hpr = pi*aux2((Vr-Vi)/(2*Vt)) + pr*aux2((Vi-Vr)/(2*Vt));
   	double hdpdxr = aux1((Vi-Vr)/(2*Vt))*(pr-pi)/h;
   	
   	double hmupr =  (mu0p(Q[0].Na,Q[0].Nd)+mu0p(Q[1].Na,Q[1].Nd))/2;
   	double hmunr =  (mu0n(Q[0].Na,Q[0].Nd)+mu0n(Q[1].Na,Q[1].Nd))/2;
   		
   	double Jn = hmunr*(hEr*hnr + Vt*hdndxr);
   	double Jp = hmupr*(hEr*hpr - Vt*hdpdxr);
   	
   	return -(Jn + Jp);
} 
